import React, { useEffect, useState } from 'react'
import { withClient, Client } from 'cozy-client'
import { translate } from 'cozy-ui/react/I18n'

import { isKonnectorRunning } from 'cozy-harvest-lib/dist/helpers/triggers'
import {
  KonnectorJob,
  ERROR_EVENT,
  LOGIN_SUCCESS_EVENT,
  SUCCESS_EVENT,
} from 'cozy-harvest-lib/dist/models/KonnectorJob'
import { Trigger } from 'doctypes'

import Lottie from 'react-lottie'
import * as loadingData from 'assets/anims/bounceloading.json'
import StyledButton from 'components/CommonKit/Button/StyledButton'
import Modal from 'components/CommonKit/Modal/Modal'

const loadingOptions = {
  loop: true,
  autoplay: true,
  animationData: loadingData,
  rendererSettings: {
    preserveAspectRatio: 'xMidYMid slice',
  },
}

interface KonnectorLaunchProps {
  trigger: Trigger
  handleKonnectorLaunch: Function
  client: Client
  t: Function
}

const KonnectorLaunch: React.FC<KonnectorLaunchProps> = ({
  trigger,
  handleKonnectorLaunch,
  client,
  t,
}: KonnectorLaunchProps) => {
  const [state, setState] = useState<string | null>(null)
  const [openModal, setOpenModal] = useState(false)

  const callbackResponse = (_state: string) => {
    setState(_state)
  }

  const handleClick = () => {
    handleKonnectorLaunch()
  }

  const handleCloseClick = () => {
    setOpenModal(false)
  }

  useEffect(() => {
    let subscribed = true
    async function getData() {
      if (subscribed && !isKonnectorRunning(trigger)) {
        const konnectorJob = new KonnectorJob(client, trigger)
        await konnectorJob.launch()
        konnectorJob.jobWatcher.on(ERROR_EVENT, () => {
          callbackResponse(ERROR_EVENT)
        })
        konnectorJob.jobWatcher.on(LOGIN_SUCCESS_EVENT, () => {
          callbackResponse(LOGIN_SUCCESS_EVENT)
        })
        konnectorJob.jobWatcher.on(SUCCESS_EVENT, () => {
          callbackResponse(SUCCESS_EVENT)
        })
        setOpenModal(true)
      }
    }
    getData()
    return () => {
      subscribed = false
    }
  }, [])

  return (
    <div className="klaunch-content">
      {!state ? (
        <>
          <Lottie options={loadingOptions} height={50} width={50} />
          <div className="klaunch-content-text klaunch-content-text-center text-16-normal">
            <div>{t('KONNECTORCONFIG.PLZ_WAIT')}</div>
          </div>
          <div className="klaunch-content-text text-16-normal">
            <div>{t('KONNECTORCONFIG.LOADING_DATA')}</div>
          </div>
        </>
      ) : (
        <>
          <Modal open={openModal} handleCloseClick={handleCloseClick} border>
            <div className="klaunch-info-txt">
              {state === ERROR_EVENT
                ? t('KONNECTORCONFIG.ERROR_DATA')
                : t('KONNECTORCONFIG.SUCCESS')}
              <StyledButton type="button" color="primary" onClick={handleClick}>
                <div>{t('KONNECTORCONFIG.OK')}</div>
              </StyledButton>
            </div>
          </Modal>
        </>
      )}
    </div>
  )
}

export default translate()(withClient(KonnectorLaunch))
